#include <windows.h>
#include <stdlib.h>

#include "global.h"
#include "resource.h"
#include "dwordedit.h"

static TCHAR * pParamName;
static DWORD   nValue;
static DWORD   nResValue;

#ifndef WIN32_WCE 
  #define _wtol atol
#endif

LONG Str_Atol (LPTSTR szString, BOOL bHex, LPBOOL bSuccess);

//****************************************************************************
BOOL WINAPI _dwordEditDialogHook (HWND h_wndDlg, UINT u_msg, WPARAM w_param, LPARAM l_param)
{
  #define DIGIT_BUFF_SIZE  40
  static TCHAR szPrevValue[DIGIT_BUFF_SIZE+1];
  static BOOL  bStateHex;
  static char  bInProcess = 1;

  TCHAR sz_dwordBuff[DIGIT_BUFF_SIZE+1];
  DWORD n_res;

  switch (u_msg)
  {
    case WM_INITDIALOG:
    {
      bInProcess = 1;
      SetDlgItemText (h_wndDlg, IDC_EDIT_PARAMETR, pParamName);
      wsprintf (szPrevValue, TEXT("%x"), nValue);
      SetDlgItemText (h_wndDlg, IDC_EDIT_VALUE, szPrevValue);
      
      CheckRadioButton (h_wndDlg, IDC_RADIO_HEX, IDC_RADIO_DEC, IDC_RADIO_HEX);
      bStateHex = TRUE;
      bInProcess = 0;
    }
    return FALSE;

    case WM_KEYDOWN:
      if (w_param == VK_DELETE)
        MessageBox (NULL, TEXT("DOWN"), TEXT("Debug"), MB_OK);
    break;

    case WM_COMMAND:
      if (LOWORD(w_param) == IDCANCEL || LOWORD(w_param) == IDOK) 
      {
        if (LOWORD(w_param) == IDOK)
        {
          GetDlgItemText (h_wndDlg, IDC_EDIT_VALUE, sz_dwordBuff, DIGIT_BUFF_SIZE);
          nResValue = Str_Atol (sz_dwordBuff, bStateHex, NULL);
        }   
        EndDialog (h_wndDlg, LOWORD(w_param));
        return LOWORD(w_param);
      }

      if (LOWORD(w_param) == IDC_RADIO_HEX && bStateHex == FALSE) //Clicked HEX and was DEC
      {
        if (HIWORD(w_param) == BN_CLICKED)
        {
          bStateHex = TRUE;
          GetDlgItemText (h_wndDlg, IDC_EDIT_VALUE, sz_dwordBuff, DIGIT_BUFF_SIZE); 

          n_res = Str_Atol (sz_dwordBuff, FALSE, NULL);

          wsprintf (szPrevValue, TEXT("%x"), n_res);
          SetDlgItemText (h_wndDlg, IDC_EDIT_VALUE, szPrevValue);
        }
      }
      else if (LOWORD(w_param) == IDC_RADIO_DEC && bStateHex == TRUE)
      {
        if (HIWORD(w_param) == BN_CLICKED)
        {
          bStateHex = FALSE;
          GetDlgItemText (h_wndDlg, IDC_EDIT_VALUE, sz_dwordBuff, DIGIT_BUFF_SIZE);

          n_res = Str_Atol (sz_dwordBuff, TRUE, NULL);

          wsprintf (szPrevValue, TEXT("%lu"), n_res);
          SetDlgItemText (h_wndDlg, IDC_EDIT_VALUE, szPrevValue);
        }
      }
      else if (LOWORD(w_param) == IDC_EDIT_VALUE)
      {
        if (HIWORD(w_param) == EN_UPDATE &&  ! bInProcess)
        {
          BOOL b_success;
          bInProcess = 1;

          GetDlgItemText (h_wndDlg, IDC_EDIT_VALUE, sz_dwordBuff, DIGIT_BUFF_SIZE);

          n_res = Str_Atol (sz_dwordBuff, bStateHex, &b_success);
          if (b_success == FALSE)
          {
            DWORD n_beg, n_end;
            SendDlgItemMessage (h_wndDlg, IDC_EDIT_VALUE, EM_GETSEL, (WPARAM)&n_beg, (LPARAM)&n_end);
            SetDlgItemText (h_wndDlg, IDC_EDIT_VALUE, szPrevValue);
            MessageBeep (MB_OK);
            SendDlgItemMessage (h_wndDlg, IDC_EDIT_VALUE, EM_SETSEL, n_beg-1, n_end-1);
          }
          else
            lstrcpy (szPrevValue, sz_dwordBuff);

          bInProcess = 0;
        }
      }
    break;
  } 
  return FALSE;
}

//****************************************************************************
DWORD DwordEdit_Run (HWND h_parentWnd, TCHAR * p_name, DWORD * p_value)
{
  DWORD ret;

  pParamName = p_name;
  nValue = *p_value;

  nResValue = 0;
  
  ret = DialogBox (hInst, MAKEINTRESOURCE(IDD_DIALOG_EDIT_DWORD), h_parentWnd,
                          (DLGPROC)(_dwordEditDialogHook));
  return ret;
}

//****************************************************************************
void DwordEdit_GetResult (DWORD ** p_result, DWORD * n_resSize)
{
  *p_result  = &nResValue;
  *n_resSize = sizeof(DWORD);
}

/************************************************************************
  Function: MyAtol(LPTSTR, BOOL, LPBOOL)
  Purpose:
    - This function will convert an ascii string to a LONG.
  Returns:
    - If it receives an invalid ascii character, it will return 0 and
      set the LPBOOL variable to false...
  Comments:
      Since the function may need to deal with either a hex number or a decimal
      number, it should use a variable as a multiplier.
************************************************************************/
LONG Str_Atol (LPTSTR szString, BOOL bHex, LPBOOL bSuccess)
{
  LPTSTR p ;
  LONG l ;
  LONG lMultiplier ;
  BOOL bDigit ;

  if (bHex)
    lMultiplier = 16 ;
  else
    lMultiplier = 10 ;

  p = szString ;
  l = 0 ;

  while (*p)      //while chars
  {
     bDigit = FALSE ;  //set to false for each char that we look at

     if (*p >= (TCHAR) '0' && *p <= (TCHAR) '9')  //is it an ascii char ?
     {
       bDigit = TRUE ;
       l+=(*p - (TCHAR) '0') ;
     }

     if (bHex)
     {
       if (*p >= (TCHAR) 'A' && *p <= (TCHAR) 'F')  //or hex?
       {
         l+=(*p - (TCHAR) 'A' + 10) ;
         bDigit = TRUE ;
       }

       if (*p >= (TCHAR) 'a' && *p <= (TCHAR) 'f') 
       {
         l+=(*p - (TCHAR) 'a' + 10) ;
         bDigit = TRUE ;
       }
     }

     if (bDigit == FALSE)
     {
       if (bSuccess != NULL) *bSuccess = FALSE ;
       return 0 ;
     }
     p++ ;               //get next char

     if (*p)             //if there is going to be at least one more char
       l*=lMultiplier ;  //then multiply what we have by the multiplier...
  }
  if (bSuccess != NULL) *bSuccess = TRUE ;
  return l ;             //success! return the value.
}
