/**********************************************************************************
MaksLand Home 1997 - 1998
=========================
Module Name:
    reglist.c

Abstract: Build list of values for a "key" and place them to 
          a ListView window

Note:
  Some parts of the code similar to source code of the REGISTRY project
  from Microsoft C++ for WCE samples (MONKEY.C)
**********************************************************************************/
#include <windows.h>
#include <commctrl.h>

#include "resource.h"
#include "global.h"
#include "stredit.h"
#include "dwordedit.h"

#include  "reglist.h"

enum EtagListIMAGES {nIMAGE_STRING, nIMAGE_MULTI_STRING, nIMAGE_DWORD, nIMAGE_BINARY};


//****************************************************************************
void RegList_LoadKey (HWND h_listWnd, HKEY h_rootKey, TCHAR * psz_subKey)
{
  HKEY   h_key;
  DWORD  dw_index;
  DWORD  n_retCode;

  DWORD  n_numSubKeys;      // address of buffer for number of subkeys 
  DWORD  n_maxSubKeyLen;    // address of buffer for longest subkey name length  
  DWORD  n_maxClassLen;     // address of buffer for longest class string length 
  DWORD  n_numValues;       // address of buffer for number of value entries 
  DWORD  n_maxValueNameLen; // address of buffer for longest value name length 
  DWORD  n_maxValueLen;     // address of buffer for longest value data length 

  TCHAR * psz_valueName;
  BYTE  * b_data;
  TCHAR * psz_outBuf; 

  LV_ITEM   listItem;

  ListView_DeleteAllItems (h_listWnd);
  //---
  if (h_rootKey == NULL)
    return;

  n_retCode = RegOpenKeyEx (h_rootKey, psz_subKey, 0, KEY_EXECUTE, &h_key);
  if (n_retCode != ERROR_SUCCESS)
  {
    TCHAR sz_message [MAX_PATH+80];
    wsprintf (sz_message, TEXT("Can't open key \'%s\'"), psz_subKey);
    MessageBox (h_listWnd, sz_message, TEXT("Error"), MB_OK|MB_ICONSTOP);
    return;
  }

  n_retCode = RegQueryInfoKey (h_key, NULL, NULL, NULL, 
                               &n_numSubKeys, &n_maxSubKeyLen, &n_maxClassLen,
                               &n_numValues, &n_maxValueNameLen, &n_maxValueLen, 
                               NULL, NULL); //&n_securityDescriptor, &ft_lastWriteTime);
  if (n_retCode != ERROR_SUCCESS || n_numValues == 0)
  {
    RegCloseKey (h_key);
    return;
  }

  n_maxValueNameLen += 1; // add 0-char

  if ((psz_valueName = (TCHAR*)LocalAlloc (LMEM_FIXED, sizeof(TCHAR)*n_maxValueNameLen)) == NULL)
  {
    MessageBox (NULL, TEXT("Error: No memory"), TEXT("Debug: RegList_LoadKey"), MB_OK);
    RegCloseKey (h_key);
    return;
  }

  if ((b_data = (BYTE*)LocalAlloc (LMEM_FIXED, n_maxValueLen)) == NULL)
  {
    MessageBox (NULL, TEXT("Error: No memory"), TEXT("Debug: RegList_LoadKey"), MB_OK);
    RegCloseKey (h_key);
    LocalFree ((HLOCAL)psz_valueName);
    return;
  }

  //---------------
  memset (&listItem, 0, sizeof (listItem));

  for (dw_index = 0; ;dw_index ++) //--- Read all sub keys 
  {
    DWORD  n_currNameLen = n_maxValueNameLen;
    DWORD  n_currDataLen = n_maxValueLen;
    DWORD  n_type; // address of buffer for type code

    n_retCode = RegEnumValue (h_key, dw_index, 
                                  psz_valueName, &n_currNameLen, 
                                  NULL,
                                  &n_type, 
                                  b_data, &n_currDataLen);
    if (n_retCode != ERROR_SUCCESS)
      break;

    switch (n_type)
    {
      case REG_SZ: case REG_EXPAND_SZ: 
        listItem.iImage = nIMAGE_STRING;
      break;

      case REG_LINK: case REG_MULTI_SZ:
        listItem.iImage = nIMAGE_MULTI_STRING;
      break;

      case REG_DWORD: case REG_DWORD_BIG_ENDIAN:
        listItem.iImage = nIMAGE_DWORD;
      break;

      case REG_BINARY: 
        listItem.iImage = nIMAGE_BINARY;
      break;
    }
    listItem.iItem = 0;
    listItem.iSubItem = 0;
    listItem.mask = LVIF_TEXT|LVIF_IMAGE;
    listItem.pszText = psz_valueName;
    n_retCode = ListView_InsertItem (h_listWnd, &listItem);
    if (n_retCode == -1)
      break;

    switch (n_type)
    {
      case REG_NONE:
        ListView_SetItemText (h_listWnd, n_retCode,	1, TEXT("REG_NONE: No defined value type."));
      break;

      case REG_SZ: case REG_EXPAND_SZ:
        psz_outBuf = (TCHAR *) LocalAlloc (LMEM_FIXED, n_currDataLen + (2*sizeof(TCHAR)));
        *psz_outBuf = 0;

        lstrcat (psz_outBuf, TEXT("\""));
        lstrcat (psz_outBuf, (LPTSTR) b_data);
        lstrcat (psz_outBuf, TEXT("\""));
        
        ListView_SetItemText (h_listWnd, n_retCode,	1, psz_outBuf);
        LocalFree ((HLOCAL)psz_outBuf);
      break;

      case REG_BINARY:
      case REG_RESOURCE_LIST: // CM_RESOURCE_LIST is complex. Print it as a free formed binary data for now.
      {
        UINT i;

        psz_outBuf = (TCHAR * )LocalAlloc (LMEM_FIXED, 3*(n_currDataLen+1) * sizeof(TCHAR));
        if (psz_outBuf == NULL)
        {
          MessageBox (h_listWnd, TEXT("Error: REG_BINARY = malloc failed"), TEXT("Debug: LoadList"), MB_OK);
          break;
        }
        for (i = 0; i < n_currDataLen; i++)
          wsprintf (psz_outBuf+(3*i), TEXT("%02x "), b_data[i]);

        psz_outBuf [i*3] = 0;
        ListView_SetItemText (h_listWnd, n_retCode,	1, psz_outBuf);
        LocalFree ((HLOCAL)psz_outBuf);
      }
      break;

      case REG_DWORD: case REG_DWORD_BIG_ENDIAN:
        psz_outBuf = (TCHAR * )LocalAlloc (LMEM_FIXED, 40 * sizeof(TCHAR));
        wsprintf (psz_outBuf, TEXT("%#08x(%lu)"), *(UINT *)b_data, *(UINT *)b_data);
        ListView_SetItemText (h_listWnd, n_retCode,	1, psz_outBuf);
        LocalFree ((HLOCAL)psz_outBuf);
      break;

      case REG_LINK:
        ListView_SetItemText (h_listWnd, n_retCode,	1, (LPTSTR) b_data);
      break;

      case REG_MULTI_SZ:
      {
        TCHAR * ptr;
        UINT    i, n_strLen;

        // Count the NULLs in the buffer to find out how many strings there are.
        for (i=0, n_strLen=4; i < n_currDataLen; i++)
          if (!b_data[i])
            n_strLen += 4; // Add room for two quotes and two spaced per string.

        psz_outBuf = (TCHAR *)LocalAlloc (LMEM_FIXED, n_currDataLen + (n_strLen + 3)*sizeof(TCHAR));

        ptr = (LPTSTR) b_data; // Set ptr to beginning of buffer.
        *psz_outBuf = 0;              // Initialize output string.

        lstrcat (psz_outBuf, TEXT("{ "));// Do first bracket.
        while (*ptr)  // Loop til you hit 2 NULLs in a row.
        {
          lstrcat (psz_outBuf, TEXT("\""));    // Put quotes around each string.
          lstrcat (psz_outBuf, ptr);
          lstrcat (psz_outBuf, TEXT("\"  "));
          ptr += lstrlen(ptr)+1;
        }
        lstrcat (psz_outBuf, TEXT("}"));      // Add final bracket.
        ListView_SetItemText (h_listWnd, n_retCode,	1, psz_outBuf);
        LocalFree ((HLOCAL) psz_outBuf);      // free output string.
      }
      break;

      case REG_FULL_RESOURCE_DESCRIPTOR:
        ListView_SetItemText (h_listWnd, n_retCode,	1, TEXT("REG_FULL_RESOURCE_DESCRIPTOR: A resource list in the hardware description."));
      break;
  
      default:
        psz_outBuf = (TCHAR * )LocalAlloc (LMEM_FIXED, 120 * sizeof(TCHAR));
        wsprintf (psz_outBuf, TEXT("Undefine in this verion of the Registry Editor. %d"), n_type);
        ListView_SetItemText (h_listWnd, n_retCode,	1, psz_outBuf);
        LocalFree ((HLOCAL) psz_outBuf);      // free output string.
      break;
    } // end switch
  }

  LocalFree ((HLOCAL)b_data);
  LocalFree ((HLOCAL)psz_valueName);
  RegCloseKey (h_key);
}

//********************************************************************************
//  FUNCTION:   RegList_EditSelectedValue (HWND h_listWnd, HKEY h_rootKey, TCHAR * psz_subKey)
//
//  PURPOSE:    
//  PARAMETERS:
//  RETURN VALUE: NONE
//  COMMENTS:
void RegList_EditSelectedValue (HWND h_listWnd, HKEY h_rootKey, TCHAR * psz_subKey)
{
  HKEY      h_key;
  LV_ITEM   listItem;
  int       n_item;
  DWORD     n_type, n_ret;
  UINT      is_selected;
  TCHAR	    sz_valName [MAX_PATH*8];

  BYTE    * p_data = NULL, * p_result = NULL;
  DWORD     n_dataSize, n_resSize;


  is_selected = ListView_GetSelectedCount (h_listWnd);
  if (! is_selected)
    return;

  n_item = ListView_GetNextItem (h_listWnd, -1, LVNI_ALL|LVNI_SELECTED);

  if (n_item < 0)
    return;

  memset (&listItem, 0, sizeof (listItem));

  listItem.pszText = sz_valName;
  listItem.cchTextMax = MAX_PATH*7;
  listItem.iItem = n_item;
  listItem.mask = LVIF_TEXT;
  if (ListView_GetItem (h_listWnd, &listItem) == FALSE)
    return;

  if (RegOpenKeyEx (h_rootKey, psz_subKey, 0, KEY_WRITE, &h_key) != ERROR_SUCCESS)
    return;

  n_ret = RegQueryValueEx (h_key,        // handle of key to query 
                           sz_valName,   // address of name of value to query 
                           NULL,         // reserved 
                           &n_type,      // address of buffer for value type 
                           NULL,         // address of data buffer 
                           &n_dataSize); // address of data buffer size 
  if (n_ret != ERROR_SUCCESS)
    return;

  if ((p_data = (BYTE*)LocalAlloc (LMEM_FIXED, n_dataSize)) == NULL)
    return;

  if (RegQueryValueEx (h_key, sz_valName, NULL, NULL, p_data, &n_dataSize) != ERROR_SUCCESS)
  {
    LocalFree ((HLOCAL)p_data);
    RegCloseKey (h_key);
    return;
  }  

  if (n_type == REG_SZ || n_type == REG_EXPAND_SZ)
  {
    if (StrEdit_Run (h_listWnd, sz_valName, (TCHAR*)p_data) == IDOK)
      StrEdit_GetResult ((TCHAR**)&p_result, &n_resSize);
  }
  else if (n_type == REG_DWORD || n_type == REG_DWORD_BIG_ENDIAN)
  {
    if (DwordEdit_Run (h_listWnd, sz_valName, (DWORD*)p_data) == IDOK)
      DwordEdit_GetResult ((DWORD**)&p_result, &n_resSize);
  }

  if (p_result != NULL)
  {
    if (RegSetValueEx (h_key,       // handle of key to set value for 
                       sz_valName,  // address of value to set 
                       0,           // reserved 
                       n_type,      // flag for value type 
                       p_result,    // address of value data 
                       n_resSize) == ERROR_SUCCESS)  // size of value data 
    {
      TCHAR * psz_buff=NULL, psz_dwordBuff[40];
      if (n_type == REG_DWORD || n_type == REG_DWORD_BIG_ENDIAN)
      {
        wsprintf (psz_dwordBuff, TEXT("%#08x(%lu)"), *(DWORD *)p_result, *(DWORD *)p_result);
        psz_buff = psz_dwordBuff;
      }
      else if (n_type == REG_SZ || n_type == REG_EXPAND_SZ)
        psz_buff = (TCHAR*)p_result;

      ListView_SetItemText (h_listWnd, n_item,	1, (TCHAR*)psz_buff);
    }
  }
  LocalFree ((HLOCAL)p_data);
  RegCloseKey (h_key);
}
