/**********************************************************************************
MaksLand Home 1997 - 1998
=========================
Module Name:
    tree.c

Abstract: TreeView operations

Note:
  Some parts of the code similar to source code of the TREEDIR project
  from Microsoft C++ for WCE samples
**********************************************************************************/
#include <windows.h>
#include <commctrl.h>
#include <memory.h>

#include "global.h"

#include "tree.h"
//==============================================================================
//  FUNCTION:   Tree_AddItem (HWND, LPSTR, int, int)
//
//  PURPOSE:    Here is where the item is actually inserted into the tree.  The
//              position on the tree is also determined.
//
//  PARAMETERS:
//      hwndTV     - handle of the treeview to add the item to
//      lpszItem   - string to add to the tree
//      htiParent  - handle of the tree item that will be this item's parent
//      fDirectory - TRUE if this item is a directory
//
//  RETURN VALUE:
//      (HTREEITEM) Returns the handle of the item once it's been added to the
//                  tree. Otherwise it returns NULL.

HTREEITEM Tree_AddItem (HWND hwndTV, LPTSTR lpszItem, HTREEITEM htiParent, BOOL fDirectory)
{
  TV_ITEM          tvi;
  TV_INSERTSTRUCT  tvins;

  // Start by initializing the structures
  memset (&tvi,   0, sizeof(TV_ITEM));
  memset (&tvins, 0, sizeof(TV_INSERTSTRUCT));

  tvi.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE;

  // If it's a directory, add a child count so the expand button shows
  if (fDirectory)
  {
    tvi.mask |= TVIF_CHILDREN;
    tvi.cChildren = 1;
  }

  // Set the text of the item
  tvi.pszText = lpszItem;
  //tvi.cchTextMax = lstrlen (lpszItem);

  tvins.item = tvi;
  tvins.hInsertAfter = TVI_SORT;

  // Set the parent item based on the specified level
  // and Give the item the appropriate image
  if (!htiParent)
  {
    tvins.hParent = TVI_ROOT;
    tvi.iSelectedImage = tvi.iImage = nIMAGE_COMPUTER; 
  }
  else
  {
    tvins.hParent = htiParent;
    tvi.iSelectedImage = tvi.iImage = nIMAGE_CLOSED; 
  }

  // Add the item to the tree view control
  tvi.hItem = (HTREEITEM) SendMessage (hwndTV, TVM_INSERTITEM, 0, (LPARAM)(LPTV_INSERTSTRUCT) &tvins);
  TreeView_SetItem (hwndTV, &tvi);

  // Return the handle to the item
  return tvi.hItem;
}
